==> idf_build_apps-1.0.0-py2.py3-none-any.whl: idf_build_apps/utils.py <==
# SPDX-FileCopyrightText: 2022-2023 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0

import fnmatch
import os
import shutil
import subprocess
import sys
from copy import (
    deepcopy,
)
from pathlib import (
    Path,
)

from . import (
    LOGGER,
)

try:
    import typing as t
except ImportError:
    pass


class ConfigRule:
    def __init__(self, file_name, config_name):  # type: (str, str) -> None
        """
        ConfigRule represents the sdkconfig file and the config name.

        For example:
            - filename='', config_name='default' — represents the default app configuration, and gives it a name
                'default'
            - filename='sdkconfig.*', config_name=None - represents the set of configurations, names match the wildcard
                value

        :param file_name: name of the sdkconfig file fragment, optionally with a single wildcard ('*' character).
            can also be empty to indicate that the default configuration of the app should be used
        :param config_name: name of the corresponding build configuration, or None if the value of wildcard is to be
            used
        """

        self.file_name = file_name
        self.config_name = config_name


def config_rules_from_str(rule_strings):  # type: (list[str] | str) -> list[ConfigRule]
    """
    Helper function to convert strings like 'file_name=config_name' into `ConfigRule` objects

    :param rule_strings: list of rules as strings or a single rule string
    :type rule_strings: list[str] | str
    :return: list of ConfigRules
    :rtype: list[ConfigRule]
    """
    if not rule_strings:
        return []

    rules = []
    for rule_str in to_list(rule_strings):
        items = rule_str.split('=', 2)
        rules.append(ConfigRule(items[0], items[1] if len(items) == 2 else None))
    # '' is the default config, sort this one to the front
    return sorted(rules, key=lambda x: x.file_name)


def get_parallel_start_stop(total, parallel_count, parallel_index):  # type: (int, int, int) -> (int, int)
    """
    Calculate the start and stop indices for a parallel task (1-based).

    :param total: total number of tasks
    :type total: int
    :param parallel_count: number of parallel tasks to run
    :type parallel_count: int
    :param parallel_index: index of the parallel task to run
    :type parallel_index: int
    :return: start and stop indices, [start, stop]
    :rtype: (int, int)
    """
    if parallel_count == 1:
        return 1, total

    num_builds_per_job = (total + parallel_count - 1) // parallel_count

    _min = num_builds_per_job * (parallel_index - 1) + 1
    _max = min(num_builds_per_job * parallel_index, total)

    return _min, _max


class BuildError(RuntimeError):
    pass


class InvalidCommand(SystemExit):
    def __init__(self, msg):
        super(InvalidCommand, self).__init__('Invalid Command: ' + msg.strip())


def rmdir(path, exclude_file_patterns=None):
    if not exclude_file_patterns:
        shutil.rmtree(path, ignore_errors=True)
        return

    for root, dirs, files in os.walk(path, topdown=False):
        for f in files:
            fp = os.path.join(root, f)
            remove = True
            for pattern in to_list(exclude_file_patterns):
                if pattern and fnmatch.fnmatch(f, pattern):
                    remove = False
                    break
            if remove:
                os.remove(fp)
        for d in dirs:
            try:
                os.rmdir(os.path.join(root, d))
            except OSError:
                pass


def find_first_match(pattern, path):
    for root, _, files in os.walk(path):
        res = fnmatch.filter(files, pattern)
        if res:
            return os.path.join(root, res[0])
    return None


def subprocess_run(
    cmd,  # type: list[str]
    log_terminal=True,  # type: bool
    log_fs=None,  # type: t.TextIO | None
    check=False,  # type: bool
    additional_env_dict=None,  # type: dict[str, any] | None
):  # type: (...) -> int
    """
    Subprocess.run for older python versions

    :param cmd: cmd
    :type cmd: list[str]
    :param log_terminal: print to `sys.stdout` if set to `True`
    :type log_terminal: bool
    :param log_fs: write to this file stream if not `None`
    :type log_fs: TextIO
    :param check: raise `BuildError` when return code is non-zero
    :type check: bool
    :param additional_env_dict: additional environment variables
    :type additional_env_dict: dict[str, any] | None
    :return: return code
    :rtype: int
    """
    LOGGER.debug('==> Running %s', ' '.join(cmd))

    subprocess_env = None
    if additional_env_dict is not None:
        subprocess_env = deepcopy(os.environ)
        subprocess_env.update(additional_env_dict)

    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=subprocess_env)
    for line in p.stdout:
        if isinstance(line, bytes):
            line = line.decode('utf-8')

        if log_terminal:
            sys.stdout.write(line)

        if log_fs:
            log_fs.write(line)

    returncode = p.wait()
    if check and returncode != 0:
        raise BuildError('Command {} returned non-zero exit status {}'.format(cmd, returncode))

    return returncode


def to_list(s):
    """
    Turn all objects to lists

    :param s: anything
    :type s: any
    :return: ``list(s)``, if ``s`` is a tuple or a set
    :return: itself, if ``s`` is a list
    :return: ``[s]``, if ``s`` is other type
    :return: ``None``, if ``s`` is None
    :rtype: list | None
    """
    if s is None:
        return s

    if isinstance(s, set) or isinstance(s, tuple):
        return list(s)
    elif isinstance(s, list):
        return s
    else:
        return [s]


def to_absolute_path(s, rootpath=None):  # type: (str, str | None) -> Path
    rp = Path(os.path.expanduser(rootpath or '.')).resolve()

    sp = Path(os.path.expanduser(s))
    if sp.is_absolute():
        return sp.resolve()
    else:
        return (rp / sp).resolve()


def files_matches_patterns(
    files,  # type: list[str] | str
    patterns,  # type: list[str] | str
    rootpath=None,  # type: str
):  # type: (...) -> bool
    # can't match an absolute pattern with a relative path
    # change all to absolute paths
    files = [to_absolute_path(f, rootpath) for f in to_list(files)]
    patterns = [to_absolute_path(p, rootpath) for p in to_list(patterns)]

    for f in files:
        for p in patterns:
            if f.match(str(p)):
                return True

    return False

==> idf_build_apps-1.0.2-py2.py3-none-any.whl: idf_build_apps/utils.py <==
# SPDX-FileCopyrightText: 2022-2023 Espressif Systems (Shanghai) CO LTD
# SPDX-License-Identifier: Apache-2.0

import fnmatch
import os
import shutil
import subprocess
import sys
from copy import (
    deepcopy,
)
from pathlib import (
    Path,
)

from packaging.version import (
    Version,
)

from . import (
    LOGGER,
)

try:
    import typing as t
except ImportError:
    pass


if sys.version_info < (3, 5):
    import glob2 as glob
else:
    import glob


class ConfigRule:
    def __init__(self, file_name, config_name):  # type: (str, str) -> None
        """
        ConfigRule represents the sdkconfig file and the config name.

        For example:
            - filename='', config_name='default' - represents the default app configuration, and gives it a name
                'default'
            - filename='sdkconfig.*', config_name=None - represents the set of configurations, names match the wildcard
                value

        :param file_name: name of the sdkconfig file fragment, optionally with a single wildcard ('*' character).
            can also be empty to indicate that the default configuration of the app should be used
        :param config_name: name of the corresponding build configuration, or None if the value of wildcard is to be
            used
        """

        self.file_name = file_name
        self.config_name = config_name


def config_rules_from_str(rule_strings):  # type: (list[str] | str) -> list[ConfigRule]
    """
    Helper function to convert strings like 'file_name=config_name' into `ConfigRule` objects

    :param rule_strings: list of rules as strings or a single rule string
    :type rule_strings: list[str] | str
    :return: list of ConfigRules
    :rtype: list[ConfigRule]
    """
    if not rule_strings:
        return []

    rules = []
    for rule_str in to_list(rule_strings):
        items = rule_str.split('=', 2)
        rules.append(ConfigRule(items[0], items[1] if len(items) == 2 else None))
    # '' is the default config, sort this one to the front
    return sorted(rules, key=lambda x: x.file_name)


def get_parallel_start_stop(total, parallel_count, parallel_index):  # type: (int, int, int) -> (int, int)
    """
    Calculate the start and stop indices for a parallel task (1-based).

    :param total: total number of tasks
    :type total: int
    :param parallel_count: number of parallel tasks to run
    :type parallel_count: int
    :param parallel_index: index of the parallel task to run
    :type parallel_index: int
    :return: start and stop indices, [start, stop]
    :rtype: (int, int)
    """
    if parallel_count == 1:
        return 1, total

    num_builds_per_job = (total + parallel_count - 1) // parallel_count

    _min = num_builds_per_job * (parallel_index - 1) + 1
    _max = min(num_builds_per_job * parallel_index, total)

    return _min, _max


class BuildError(RuntimeError):
    pass


class InvalidCommand(SystemExit):
    def __init__(self, msg):
        super(InvalidCommand, self).__init__('Invalid Command: ' + msg.strip())


class InvalidInput(SystemExit):
    """Invalid input from user"""


def rmdir(path, exclude_file_patterns=None):
    if not exclude_file_patterns:
        shutil.rmtree(path, ignore_errors=True)
        return

    for root, dirs, files in os.walk(path, topdown=False):
        for f in files:
            fp = os.path.join(root, f)
            remove = True
            for pattern in to_list(exclude_file_patterns):
                if pattern and fnmatch.fnmatch(f, pattern):
                    remove = False
                    break
            if remove:
                os.remove(fp)
        for d in dirs:
            try:
                os.rmdir(os.path.join(root, d))
            except OSError:
                pass


def find_first_match(pattern, path):
    for root, _, files in os.walk(path):
        res = fnmatch.filter(files, pattern)
        if res:
            return os.path.join(root, res[0])
    return None


def subprocess_run(
    cmd,  # type: list[str]
    log_terminal=True,  # type: bool
    log_fs=None,  # type: t.TextIO | None
    check=False,  # type: bool
    additional_env_dict=None,  # type: dict[str, any] | None
):  # type: (...) -> int
    """
    Subprocess.run for older python versions

    :param cmd: cmd
    :type cmd: list[str]
    :param log_terminal: print to `sys.stdout` if set to `True`
    :type log_terminal: bool
    :param log_fs: write to this file stream if not `None`
    :type log_fs: TextIO
    :param check: raise `BuildError` when return code is non-zero
    :type check: bool
    :param additional_env_dict: additional environment variables
    :type additional_env_dict: dict[str, any] | None
    :return: return code
    :rtype: int
    """
    LOGGER.debug('==> Running %s', ' '.join(cmd))

    subprocess_env = None
    if additional_env_dict is not None:
        subprocess_env = deepcopy(os.environ)
        subprocess_env.update(additional_env_dict)

    p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=subprocess_env)
    for line in p.stdout:
        if isinstance(line, bytes):
            line = line.decode('utf-8')

        if log_terminal:
            sys.stdout.write(line)

        if log_fs:
            log_fs.write(line)

    returncode = p.wait()
    if check and returncode != 0:
        raise BuildError('Command {} returned non-zero exit status {}'.format(cmd, returncode))

    return returncode


def to_list(s):
    """
    Turn all objects to lists

    :param s: anything
    :type s: any
    :return: ``list(s)``, if ``s`` is a tuple or a set
    :return: itself, if ``s`` is a list
    :return: ``[s]``, if ``s`` is other type
    :return: ``None``, if ``s`` is None
    :rtype: list | None
    """
    if s is None:
        return s

    if isinstance(s, set) or isinstance(s, tuple):
        return list(s)
    elif isinstance(s, list):
        return s
    else:
        return [s]


def to_absolute_path(s, rootpath=None):  # type: (str, str | None) -> Path
    rp = Path(os.path.expanduser(rootpath or '.')).resolve()

    sp = Path(os.path.expanduser(s))
    if sp.is_absolute():
        return sp.resolve()
    else:
        return (rp / sp).resolve()


def to_version(s):  # type: (any) -> Version
    if isinstance(s, Version):
        return s

    try:
        return Version(str(s))
    except ValueError:
        raise InvalidInput('Invalid version: {}'.format(s))


def files_matches_patterns(
    files,  # type: list[str] | str
    patterns,  # type: list[str] | str
    rootpath=None,  # type: str
):  # type: (...) -> bool
    # can't match an absolute pattern with a relative path
    # change all to absolute paths
    files = [to_absolute_path(f, rootpath) for f in to_list(files)]
    patterns = [to_absolute_path(p, rootpath) for p in to_list(patterns)]

    matched_paths = set()
    for pat in patterns:
        matched_paths.update(glob.glob(str(pat), recursive=True))

    for f in files:
        if str(f) in matched_paths:
            return True

    return False
